/*
 *  dumpfile.c
 *
 *  Copyright (C) 2005  FUJITSU LIMITED
 *
 */
/* $Id: dumpfile.c,v 1.2 2005/11/09 19:48:31 tachino Exp $ */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "savecore.h"

DumpFile *open_dump_file(char *dir)
{
	DumpFile *dump_file;
	char *filename;
	int fd;

	dump_file = g_new0(DumpFile, 1);

	dump_file->dir = dir;

	filename = g_strconcat(dir, INCOMPLETE_DUMP_FILE, NULL);

	fd = open(filename, O_WRONLY|O_TRUNC|O_CREAT, S_IRUSR|S_IWUSR);
	if (fd < 0) {
		g_free(dump_file);
		g_free(filename);
		return NULL;
	}

	dump_file->fd	= fd;
	dump_file->name	= filename;

	return dump_file;
}

int rename_dump_file(DumpFile *dump_file)
{
	char *old_file;
	char *new_file;
	int ret;

	old_file = g_strconcat(dump_file->dir, INCOMPLETE_DUMP_FILE, NULL);
	new_file = g_strconcat(dump_file->dir, COMPLETE_DUMP_FILE, NULL);

	ret = rename(old_file, new_file);
	dump_file->name = g_strdup(new_file);

	g_free(old_file);
	g_free(new_file);

	return ret;
}

int write_page_desc(DumpFile *dump_file, page_desc_t *desc)
{
	if (dump_seek(dump_file, dump_file->page_desc_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, desc, sizeof(*desc)) != sizeof(*desc))
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->page_desc_offset += sizeof(*desc);

	return COMPLETE;
}

int write_page_data(DumpFile *dump_file, const void *data, uint32_t size)
{
	if (dump_seek(dump_file, dump_file->data_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, data, size) != size)
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->data_offset += size;

	return COMPLETE;
}

int write_blocks(DumpFile *dump_file, const void *data, uint32_t size)
{
	int block_size = dump_file->device->header->block_size;

	if (dump_seek(dump_file, dump_file->data_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, data, size) != size)
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->data_offset += roundup(size, block_size);

	return COMPLETE;
}
